@@ -116,8 +116,11 @@
 #'   from multiple tissues samples across the same individuals. If the dataset 
 #'   specified is the \code{discovery} dataset, then missing samples will be 
 #'   displayed as horizontal grey bars. If the dataset specified is one of the 
-#'   other datasets, then only samples present in both the specified dataset 
-#'   and the \code{test} dataset will be displayed.
+#'   other datasets, samples present in both the specified dataset and the 
+#'   \code{test} dataset will be displayed first in order of the specified 
+#'   dataset, then samples present in only the test dataset will be displayed
+#'   underneath a horizontal black line ordered by their module summary vector 
+#'   in the test dataset.
 #'    
 #'   Order of samples by \emph{module summary} can be suppressed by setting 
 #'   \code{orderSamplesBy} to \code{NA}, in which case samples will be order as
@@ -341,6 +344,10 @@ plotModule <- function(
   netRange=c(0,1), degreeCol="#feb24c", contribCols=c("#A50026", "#313695"), 
   summaryCols=c("#1B7837", "#762A83"), naCol="#bdbdbd", dryRun=FALSE
 ) {
+  # always garbage collect before the function exits so any loaded 
+  # disk.matrices get unloaded as appropriate
+  on.exit({ gc() }, add = TRUE) 
+  
   #-----------------------------------------------------------------------------
   # Set graphical parameters to catch errors prior to computation
   #-----------------------------------------------------------------------------
@@ -399,14 +406,15 @@ plotModule <- function(
   loadedIdx <- finput$loadedIdx
   
   # Get the loaded datasets
-  dataLoaded <- finput$dataLoaded
-  correlationLoaded <- finput$correlationLoaded
-  networkLoaded <- finput$networkLoaded
-  # remove from the finput list so that when we re-assign a new dataset the
-  # memory is freed.
-  finput$dataLoaded <- NULL
-  finput$correlationLoaded <- NULL
-  finput$networkLoaded <- NULL
+  dataEnv <- finput$dataEnv
+  correlationEnv <- finput$correlationEnv
+  networkEnv <- finput$networkEnv
+  
+  # We don't want a second copy of these environments when we start 
+  # swapping datasets.
+  finput$dataEnv <- NULL
+  finput$correlationEnv <- NULL
+  finput$networkEnv <- NULL
   
   # Flag for on.exit
   anyDM <- any.disk.matrix(data[[loadedIdx]], correlation[[loadedIdx]], 
@@ -414,7 +422,7 @@ plotModule <- function(
   
   on.exit({
     vCat(verbose && anyDM, 0, "Unloading dataset from RAM...")
-    rm(dataLoaded, correlationLoaded, networkLoaded)
+    rm(dataEnv, correlationEnv, networkEnv)
     gc()
   }, add=TRUE)
   
@@ -470,7 +478,7 @@ plotModule <- function(
   
   plotProps <- plotProps(network, data, moduleAssignments, modules, di, ti, 
     orderNodesBy, orderSamplesBy, orderModules, datasetNames, nDatasets, 
-    verbose, loadedIdx, as.ref(dataLoaded), as.ref(networkLoaded))
+    verbose, loadedIdx, dataEnv, networkEnv)
   testProps <- plotProps$testProps
   nodeOrder <- plotProps$nodeOrder
   moduleOrder <- plotProps$moduleOrder
@@ -479,6 +487,7 @@ plotModule <- function(
   na.pos.y <- plotProps$na.pos.y
   presentNodes <- plotProps$presentNodes
   presentSamples <- plotProps$presentSamples
+  nNewSamples <- plotProps$nNewSamples
   
   # flag for on.exit
   anyDM <- any.disk.matrix(data[[ti]], correlation[[ti]], network[[ti]])
@@ -487,9 +496,9 @@ plotModule <- function(
     vCat(verbose && is.disk.matrix(correlation[[ti]]), 0, 
          'Loading correlation matrix of dataset "', ti, '" into RAM...', 
          sep="")
-    rm(correlationLoaded)
+    correlationEnv$matrix <- NULL
     gc()
-    correlationLoaded <- loadIntoRAM(correlation[[ti]])
+    correlationEnv$matrix <- loadIntoRAM(correlation[[ti]])
   }
   
   #-----------------------------------------------------------------------------
@@ -501,7 +510,7 @@ plotModule <- function(
     # placehold vectors so that the plot functions work
     wDegreeVec <- rep(0, length(nodeOrder))
     names(wDegreeVec) <- nodeOrder
-    if (!is.null(dataLoaded)) {
+    if (!is.null(dataEnv$matrix)) {
       nodeContribVec <- rep(0, length(nodeOrder))
       names(nodeContribVec) <- nodeOrder
       
@@ -562,26 +571,43 @@ plotModule <- function(
     dimnames(dat) <- list(presentSamples, presentNodes)
     if (is.null(dataRange)) {
       dataRange <- c(-1, 1)
-    }
+    } 
+    dataLegendRange <- dataRange
   } else {
-    dat <- dataLoaded[presentSamples, presentNodes] # also used for actual plot
-    if (is.null(dataRange)) {
-      dataRange <- range(dat)
-      # Make sure the gradient is balanced around 0 if the default colors are
-      # requested
-      if (is.null(dataCols) && dataRange[1] < 0 && dataRange[2] > 0) {
-        dataRange <- c(-1*max(abs(dataRange)), max(abs(dataRange)))
-      }
+    # dataRange - what range of values are present in the data matrix?
+    # dataLegendRange - what range of values do you want to show on the gradient legend?
+    # dataGradientRange - what range of values should the gradient span (e.g. in the case
+    #                     of positive and negative values to keep white at 0)
+    
+    dat <- dataEnv$matrix[presentSamples, presentNodes, drop=FALSE] # also used for actual plot
+    
+    # The user input 'dataRange' is more accurately 'dataLegendRange' but I don't 
+    # want to change the API now the package is released
+    dataLegendRange <- dataRange
+    dataRange <- range(dat)
+    
+    # If unspecified, set automatically to the range of the data
+    if (is.null(dataLegendRange)) {
+      dataLegendRange <- dataRange
     }
   }
+  
+  # Choose color palette if not specified and set the range of values the palette
+  # should span
   if (is.null(dataCols)) {
-    if (all(dataRange >= 0)) {
+    if (all(dataLegendRange >= 0)) {
       dataCols <- c("#FFFFFF", "#1B7837")
-    } else if (all(dataRange <= 0)) {
+      dataGradientRange <- dataLegendRange
+    } else if (all(dataLegendRange <= 0)) {
       dataCols <- c("#762A83", "#FFFFFF")
+      dataGradientRange <- dataLegendRange
     } else {
       dataCols <- c("#762A83", "#FFFFFF", "#1B7837")
+      # Make sure the gradient is balanced around 0!
+      dataGradientRange <- c(-1*max(abs(dataLegendRange)), max(abs(dataLegendRange)))
     }
+  } else {
+    dataGradientRange <- dataLegendRange
   }
   
   naxt <- NULL
@@ -595,7 +621,7 @@ plotModule <- function(
   # Plot correlation
   par(mar=c(1, 1, 1, 1))
   plotTriangleHeatmap(
-    correlationLoaded[presentNodes, presentNodes], 
+    correlationEnv$matrix[presentNodes, presentNodes, drop=FALSE], 
     corCols, corRange, moduleAssignments[[di]][nodeOrder], na.pos.x, 
     plotLegend=TRUE, main=main, main.line=main.line, legend.main="Correlation", 
     plotModuleNames=FALSE, laxt.tck=laxt.tck, laxt.line=laxt.line, na.col=naCol,
@@ -606,7 +632,7 @@ plotModule <- function(
   # Plot network
   par(mar=c(1, 1, 1, 1))
   plotTriangleHeatmap(
-    networkLoaded[presentNodes, presentNodes], netCols, netRange,
+    networkEnv$matrix[presentNodes, presentNodes, drop=FALSE], netCols, netRange,
     moduleAssignments[[di]][nodeOrder], na.pos.x, plotLegend=TRUE, main="", 
     legend.main="Edge weights", plotModuleNames=FALSE, 
     laxt.tck=laxt.tck, na.col=naCol, legend.main.line=legend.main.line,
@@ -616,7 +642,7 @@ plotModule <- function(
   
   # Plot weighted degree
   par(mar=c(1,1,1,1))
-  if (is.null(dataLoaded)) {
+  if (is.null(dataEnv$matrix)) {
     plotBar(
       wDegreeVec, c(0,1), moduleAssignments[[di]][nodeOrder], degreeCol, 
       drawBorders=drawBorders, plotModuleNames=plotModuleNames, 
@@ -634,7 +660,7 @@ plotModule <- function(
     )
   }
   
-  if (!is.null(dataLoaded)) {
+  if (!is.null(dataEnv$matrix)) {
     # Plot Module Membership
     par(mar=c(1, 1, 1, 1))
     plotBar(
@@ -654,20 +680,20 @@ plotModule <- function(
       yaxt <- sampleOrder
     par(mar=c(1, 1, 1, 1))
     plotSquareHeatmap(
-      dat, dataCols, vlim=dataRange,
+      dat, dataCols, dataGradientRange,
       moduleAssignments[[di]][nodeOrder], na.pos.x, na.pos.y, 
       xaxt=naxt, yaxt=NULL, plotLegend=FALSE, main="",
       legend.main="", plotModuleNames=plotModuleNames,
       xaxt.line=naxt.line, maxt.line=maxt.line, lwd=lwd,
-      dryRun=dryRun
+      dryRun=dryRun, yLine=nNewSamples
     )
     
     # Plot data legend
     xHalfUnit <- (1/(ncol(dat) + length(na.pos.x)))/2
     yHalfUnit <- (1/(nrow(dat) + length(na.pos.y)))/2
     addGradientLegend(
-      dataCols, dataRange, TRUE, main="Module data",
-      xlim=c(xHalfUnit + 0.1, 1 - xHalfUnit - 0.1), 
+      dataCols, dataLegendRange, dataGradientRange, dataRange,
+      TRUE, main="Module data", xlim=c(xHalfUnit + 0.1, 1 - xHalfUnit - 0.1), 
       ylim=c(1 + yHalfUnit + 0.2, 1 + yHalfUnit + 0.3),  
       tck=laxt.tck, lwd=lwd, legend.main.line=legend.main.line,
       axis.line=laxt.line, srt=0
@@ -684,8 +710,8 @@ plotModule <- function(
       yaxt=plotSampleNames, plotModuleNames=plotModuleNames, 
       yaxt.line=saxt.line, maxt.line=0, xlab=xlab, xaxt.line=xaxt.line, 
       xlab.line=xlab.line, xaxt.tck=xaxt.tck, cex.modules=par("cex.lab"), 
-      na.col=naCol, dryRun=dryRun
+      na.col=naCol, dryRun=dryRun, yLine=nNewSamples
     )
   }
   on.exit({vCat(verbose, 0, "Done!")}, add=TRUE)
-}
\ No newline at end of file
+}
